//--------------------------------------------------------------
// File     : stm32_ub_cs43l22.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_CS43L22_H
#define __STM32F4_UB_CS43L22_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "cs43l22_lolevel.h"


//--------------------------------------------------------------
// Struktur von einem WAV-File
//--------------------------------------------------------------
typedef struct UB_WAV_t
{
  const uint8_t *table; // Tabelle mit den Daten
  uint32_t size;        // Anzahl der Bytes
}UB_WAV;



//--------------------------------------------------------------
// Einbinden der benutzen WAV-Files
// fuer alle WAV-Files muss das entsprechende File
// auch in der CooCox-IDE hinzugefgt werden
//--------------------------------------------------------------
//extern UB_WAV s16k16b_wav;
extern UB_WAV m32k16b_wav;


//--------------------------------------------------------------
// interne Enum
//--------------------------------------------------------------
typedef enum {
  CS43L22_NO_INIT = 0,
  CS43L22_INIT_OK,
  CS43L22_PLAY,
  CS43L22_PAUSE,
  CS43L22_STOP
}CS43L22_STATUS_t;


//--------------------------------------------------------------
// interne Struktur
//--------------------------------------------------------------
typedef struct {
  uint32_t file_len;
  uint32_t fmt_len;
  uint16_t typ;
  uint16_t channels;
  uint32_t samplerate;
  uint32_t byterate;
  uint16_t framesize;
  uint16_t bitpersample;
  uint32_t data_len;
  uint32_t data_start;
}WAV_t;



//--------------------------------------------------------------
// Fehler beim check vom WAV-Header
//--------------------------------------------------------------
typedef enum {
  WAV_OK = 0,        // WAVE-Header ist ok
  WAV_INIT_ERR,      // Fehler beim init
  WAV_SIZE_ERR,      // File zu klein
  WAV_ID_ERR,        // RIFF-ID fehlt
  WAV_ID2_ERR,       // WAVE-ID fehlt
  WAV_ID3_ERR,       // fmt-ID fehlt
  WAV_FMT_LEN_ERR,   // falsche Lnge von ftm
  WAV_TYP_ERR,       // Typ ist nicht PCM
  WAV_CH_ERR,        // kein Mono oder Stereo
  WAV_FRQ_ERR,       // Fehler bei Samplefrq
  WAVE_FRAME_ERR,    // Fehler im Frame
  WAVE_BPS_ERR,      // Fehler bei Bit per Sample
  WAVE_ID4_ERR       // DATA-ID fehlt
}WAV_ERR_t;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_CS43L22_Init(void);
WAV_ERR_t UB_CS43L22_PlayWAVSingle(UB_WAV *wav, uint8_t Volume);
WAV_ERR_t UB_CS43L22_PlayWAVLoop(UB_WAV *wav, uint8_t Volume);
void UB_CS43L22_PauseWAV(void);
void UB_CS43L22_ResumeWAV(void);
void UB_CS43L22_StopWAV(void);
void UB_CS43L22_SetVolume(uint8_t Volume);



//--------------------------------------------------------------
#endif // __STM32F4_UB_CS43L22_H
